local assets =
{
	Asset("ANIM", "anim/base_light.zip"),
	Asset("ANIM", "anim/lightning_rod_fx.zip"),
}

local prefabs = 
{
    "lightning_rod_fx"
}

local function onhammered(inst, worker)
    inst.components.lootdropper:DropLoot()
    SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
    inst:Remove()
    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
end
        
local function onhit(inst, worker)
    inst.AnimState:PlayAnimation("hit")
    inst.AnimState:PushAnimation("idle", false)
end

local function discharge(inst)
    inst.AnimState:PlayAnimation("idle")
    inst.charged = false
    inst.chargeleft = nil
    inst.Light:Enable(false)
    if inst.zaptask then
    inst.zaptask:Cancel()
    inst.zaptask = nil
    end
end

local function dozap(inst)
    if inst.zaptask then
    inst.zaptask:Cancel()
    inst.zaptask = nil
end
    inst.SoundEmitter:PlaySound("dontstarve/common/lightningrod")
    local fx = SpawnPrefab("lightning_rod_fx")
    local pos = inst:GetPosition()
    fx.Transform:SetPosition(pos.x, pos.y, pos.z)
    inst.zaptask = inst:DoTaskInTime(math.random(10, 40), dozap)
end

local function setcharged(inst)
    dozap(inst)
    inst.AnimState:PlayAnimation("idle_on")
    inst.Light:Enable(true)
    inst.charged = true
    inst.chargeleft = TUNING.CHARGE
    inst:ListenForEvent( "daycomplete", function()
    if inst.chargeleft then
    dozap(inst)
    inst.chargeleft = inst.chargeleft - 1
    if inst.chargeleft <= 0 then
    discharge(inst)
            end
        end
    end, GetWorld())
end

local function onlightning(inst)
    onhit(inst)
    setcharged(inst)
end

local function OnSave(inst, data)
    if inst.charged then
    data.charged = inst.charged
    data.chargeleft = inst.chargeleft
    end
end

local function OnLoad(inst, data)
    if data and data.charged and data.chargeleft then
    setcharged(inst)
    inst.chargeleft = data.chargeleft
    end
end

local function getstatus(inst)
    if inst.charged then
    return "CHARGED"
    end
end

local function onbuilt(inst)
    inst.AnimState:PlayAnimation("place")
    inst.AnimState:PushAnimation("idle")
end

local function fn(Sim)
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState() 
 
    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "base_light.tex" )
	
    local light = inst.entity:AddLight()
    inst.Light:Enable(false)
    inst.Light:SetRadius(12)
    inst.Light:SetFalloff(.9)
    inst.Light:SetIntensity(.6)
    inst.Light:SetColour(169/255, 231/255, 245/255)

    inst.entity:AddSoundEmitter()
    inst:AddTag("structure")
    inst:AddTag("lightningrod")

    anim:SetBank("base_light")
    anim:SetBuild("base_light")
    anim:PlayAnimation("idle")

    inst:ListenForEvent("lightningstrike", onlightning)
    inst:AddComponent("lootdropper")
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
    inst.components.workable:SetOnFinishCallback(onhammered)

    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = getstatus
    
    MakeSnowCovered(inst, .01)

    inst.OnSave = OnSave
    inst.OnLoad = OnLoad
    return inst
end

return Prefab( "common/objects/base_light", fn, assets, prefabs),
	   MakePlacer("common/base_light_placer", "base_light", "base_light", "idle")  
